SystemOrganization addCategory: #AGV!
SystemOrganization addCategory: #'AGV-SIM'!

Announcement subclass: #AgvOccupied
	instanceVariableNames: 'agvNo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV'!

!AgvOccupied commentStamp: '<historical>' prior: 0!
An AgvOccupied  is an Announcement .

Instance Variables
	agvNo:		<String>
!

!AgvOccupied class methodsFor: 'instance creation' stamp: 'hk 8/24/2014 15:39'!
no: agvNo 
	^self new agvNo: agvNo! !

!AgvOccupied methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:22'!
agvNo
	^ agvNo! !

!AgvOccupied methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:22'!
agvNo: anObject
	agvNo := anObject! !

!AgvOccupied methodsFor: 'converting' stamp: 'hk 8/24/2014 14:31'!
asString
	^ 'AGV no: ', agvNo, ' is occupied'! !

Announcement subclass: #TaskFinished
	instanceVariableNames: 'agvNo taskNo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV'!

!TaskFinished commentStamp: '<historical>' prior: 0!
A TaskFinished is an Announcement .

Instance Variables
	agvNo:		<String>
	taskNo:		<String>
!

!TaskFinished class methodsFor: 'instance creation' stamp: 'hk 8/24/2014 14:16'!
for: taskNo on: agvNo
	^self new 
		taskNo: taskNo; 
		agvNo: agvNo! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
agvNo
	^ agvNo! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
agvNo: anObject
	agvNo := anObject! !

!TaskFinished methodsFor: 'converting' stamp: 'hk 8/24/2014 14:31'!
asString
	^ 'task ', taskNo, ' for AGV ', agvNo, ' is finished'! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
taskNo
	^ taskNo! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
taskNo: anObject
	taskNo := anObject! !

Object subclass: #AGV
	instanceVariableNames: 'x v ambit task target name1 locked'
	classVariableNames: 'A AGVs Area Flane Frep MyAnnouncer Radius Simulation Vmax Vmin'
	poolDictionaries: ''
	category: 'AGV'!

!AGV commentStamp: '<historical>' prior: 0!
An AGV moves according to forces from its environment.

Instance Variables 
	ambit:			<Coll of Points>	all objects that repel me
	v:				<Point>			m/s
	x:				<Point>			m 
	task:			<Task | nil>	
	target:			<Point | nil>		Cache for target coordinate
	name1:			<String>
	locked:			<Integer>			0 = unlocked, count down
							
Forces act between the AGV and 
i) boundary, other AGVs and other lanes - repulsive 1/r^2
ii) loadLane / unloadLane - attractive constant before visit and 
						  - repulsive after visit = i) 
Friction surrogate:
see self checkTarget and self maxV

Class variables
	AGVs 		<Dictionary of: name1 and AGV>	
	Flane 		<Number>		attractive constant force to target lane
	Frep 		<Number>		repulsive 1/r^2 force to agv & other lanes
	Vmax		<Number>		>0
	Vmin		<Number>		>0
	A			<Number>		1 > A >0
	Radius 		<Number>		of the environment
	Area 		interface object that ca be asked
				- for the ambit (environment)
				- for lane coordinates
				- to update own coordinates
				- to add or remove an AGV
	MyAnnouncer 		
				interface mediator
				
Todo: =========================================
- Tests 
!

!AGV class methodsFor: 'interface-geo' stamp: 'hk 7/2/2014 20:33'!
AGVsFromArea
	Area ifNil: [ ^self ]. 
	(Area allObjectNamesAndPositionsWith: 'agv*') keysAndValuesDo: 
		[ :k :v | AGVs add: (Association key: k value: (self named: k at: v)) ]! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 7/4/2014 16:21'!
allAGVs
	"answerds a Dictionary of the AGV names with [location, status] as value"

	^AGVs collect: [ :v | Array with: v location  with: v free  ]! !

!AGV class methodsFor: 'class initialization' stamp: 'hk 8/24/2014 15:22'!
announcer: anAnn
	MyAnnouncer := anAnn! !

!AGV class methodsFor: 'class initialization' stamp: 'hk 8/24/2014 15:18'!
initialize 
	AGVs := Dictionary new.
	Flane := 5 .
	Frep := 500 .
	Vmax := 5 .
	Vmin := 0.5 .
	A := 0.7 .
	Radius := 40 .
! !

!AGV class methodsFor: 'instance creation' stamp: 'hk 7/4/2014 16:24'!
named: aName at: aPoint 
	^(self new) 
		location: aPoint;
		name1: aName;
		yourself! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 7/3/2014 12:38'!
next
	AGVs do: [ :e |  e next ]! !

!AGV class methodsFor: 'class initialization' stamp: 'hk 7/2/2014 15:53'!
setArea: anArea 
	Area := anArea! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 8/24/2014 15:41'!
task: agvNo from: lane1 to: lane2 
	"AGV task: 'agv03' from: 'laneCB1S1'  to: 'laneLB2S3' "

	| agv |
	agv := (AGVs at: agvNo).
	agv free ifFalse: [ MyAnnouncer announce: (AgvOccupied no: agvNo)].
	agv
		newTask: lane1, lane2
		from: lane1
		to: lane2
	! !

!AGV methodsFor: 'simulation' stamp: 'hk 8/20/2014 17:16'!
checkTarget
	"loadLane / unloadLane will be reached when v=dx < vmin.
	adjust x, load or unload, switch task"
	"PRE Task isNotNil"

	(target-x) r > Vmin ifTrue: [ ^self ].			"not yet there"
	self updateX .
	v := 0@0. 
	self lock
	! !

!AGV methodsFor: 'testing' stamp: 'hk 6/30/2014 11:24'!
empty
	^ task empty! !

!AGV methodsFor: 'testing' stamp: 'hk 7/2/2014 12:13'!
finished
	^ task finished! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/4/2014 14:09'!
flushAmbit
	ambit  := Area objectLocationsWithin: Radius of:  x without:  (Array with: self targetLane with: name1). 
	ambit addAll: ( Area boundaryPointsWithin: Radius of:  x).
	 ! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/2/2014 10:58'!
flushTarget
	"to be called after task progress"
	
	target := Area locationOfObjectNamed: self targetLane 
! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/7/2014 14:42'!
forceFrom: aPoint
	"the repulsive force is directed from aPoint to x"
	"after load or unload the agv may be at the target,
	hence we avoid to divide by Zero"
	
	| d |
	d := x squaredDistanceTo:  aPoint.
	^d = 0 
		ifTrue: [ ^Frep asFloat * (x - aPoint) ]
		ifFalse: [ ^Frep asFloat * (x - aPoint) /  d ]
	 ! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 16:01'!
forceTo: aPoint
	"the attractive force is directed to aPoint"
	
	^10 * Flane asFloat * (aPoint -x) / (aPoint dist: x)! !

!AGV methodsFor: 'testing' stamp: 'hk 7/2/2014 12:09'!
free
	^task = nil! !

!AGV methodsFor: 'initialize-release' stamp: 'hk 7/8/2014 12:46'!
initialize 	
	v := 0@0.
	x := 0@0.
	locked := 0.
	ambit := #()! !

!AGV methodsFor: 'testing' stamp: 'hk 6/30/2014 11:24'!
loaded
	^ task loaded! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/2/2014 16:10'!
location
	^x! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/4/2014 16:24'!
location: aPoint 
	x := aPoint! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 12:48'!
lock
	locked := 5
	! !

!AGV methodsFor: 'testing' stamp: 'hk 7/8/2014 12:49'!
locked
	^locked > 0
	! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 16:51'!
maxV:  aV
	"the scalar v does neither exceed vmax nor a * vd, 
	the velocity needed to reach the lane in the next step; 0.5 < a < 1.
	answer a modified vector"

	((target notNil) and: [Vmax > (target - x ) r ])
		ifTrue: [^A * (target-x)].
	^(aV r < Vmax) ifTrue: [ aV ] ifFalse: [ aV * Vmax / aV r ] ! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/2/2014 13:51'!
name1: aName 
	name1 := aName! !

!AGV methodsFor: 'simulation' stamp: 'hk 8/20/2014 17:54'!
newTask: partialID from: a to: b
	"assign a new task"
	
	task := Task id: partialID from: a to: b.
	self flushTarget; updateEmpty 
! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 16:18'!
next
	self readyForSwitch ifTrue: [ ^self switchTask ].
	self locked ifTrue: [ ^self wait ].
	v := v * A.								"running out slowly"
	v := v + self sumOfForces.				"f = dv; m=dt =1"
	v := self maxV: v .						"not faster"
	x := (x + v) asNonFractionalPoint.		" move "
	self updateX.
	task ifNotNil: [self checkTarget]		" target reached? "
	! !

!AGV methodsFor: 'testing' stamp: 'hk 7/2/2014 12:12'!
occupied
	^self free not! !

!AGV methodsFor: 'testing' stamp: 'hk 7/8/2014 12:51'!
readyForSwitch
	^locked = 1
	! !

!AGV methodsFor: 'interface-simulation' stamp: 'hk 8/24/2014 14:45'!
reportTask
"	World ifNil: [ ^self ].
	World notify: name1 with: task id telling: 'finished'"
	
	MyAnnouncer announce: (TaskFinished for: task id on: name1).! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/4/2014 16:34'!
sumOfForces
	| f |
	f := 0@0.
	self flushAmbit.
	ambit do: [ :p | f := f + (self forceFrom: p) ].
	target ifNotNil: [ f := f + (self forceTo: target) ].
	^f asNonFractionalPoint! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 17:20'!
switchTask
	"load or unload, switch task"
	"PRE Task isNotNil"

	self unlock.
	task next.									"do the target action"
	self loaded ifTrue: 
		[ self flushTarget; updateLoaded ].		"unload target"
	self finished ifTrue: 
		[self reportTask; updateFree. 
		target := nil. task := nil ].
	! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/2/2014 12:03'!
targetLane
	self free	ifTrue: [ ^'none' ].
	self empty 	ifTrue: [ ^task loadLane].
	self loaded ifTrue: [ ^task unloadLane  ]! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 12:54'!
unlock
	locked := 0
	! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/7/2014 15:06'!
updateEmpty
	Area ifNotNil: [ Area change: name1 at: #color: to: Color yellow]! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/8/2014 17:19'!
updateFree
	Area ifNotNil: [ Area change: name1 at: #color: to: Color lightGreen]! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/2/2014 13:41'!
updateLoaded
	Area ifNotNil: [ Area change: name1 at: #color: to: Color red]! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/2/2014 13:28'!
updateX
	Area ifNotNil: [ Area  moveObject: name1 To: x ]! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 12:48'!
wait
	locked := locked - 1
	! !

Object subclass: #AgvConfigurator
	instanceVariableNames: 'dispatcher visualisation agvUI geoWindow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV-SIM'!

!AgvConfigurator commentStamp: '<historical>' prior: 0!
dock addMorphBack: self simulationToolbar .
	dock addMorphBack: self agvToolbar .
An AgvConfigurator is the component that configures the agv (simulation) system: init and start-up.

Instance Variables
	dispatcher:		<AgvDispatcher>
	visualisation:	<GeoDSLDrawing>
	geoWindow:	<StandardWindow>  shows the Terminal Layout
	agv:			global class, no slot needed

Todo: =========================================
- Tests 
- UI mit start/stop  und Auftrgen (Tasks)!

!AgvConfigurator class methodsFor: 'example' stamp: 'hk 8/23/2014 13:59'!
run
	"AgvConfigurator run"
	AgvConfigurator new! !

!AgvConfigurator methodsFor: 'accessing' stamp: 'hk 8/22/2014 18:54'!
dispatcher
	^ dispatcher! !

!AgvConfigurator methodsFor: 'initialize-release' stamp: 'hk 8/24/2014 20:13'!
initialize 
	"AgvConfigurator new "
	
	| geoModel ann |
	AGV initialize; announcer: (ann := Announcer new).
	Task initialize.
	
	geoModel := GeoDSLModel new.	 
	visualisation := geoModel drawingFrom: self simpleTerminalAGVArea.
	AGV setArea: visualisation; AGVsFromArea.	
	(dispatcher := AgvDispatcher new) getAgvs .
	AgvUI announcer: ann.
	agvUI := AgvUI on: self and: dispatcher. 

	geoWindow := geoModel open.
	agvUI open.
! !

!AgvConfigurator methodsFor: 'initialize-release' stamp: 'hk 8/18/2014 16:03'!
release 
geoWindow close.
dispatcher release! !

!AgvConfigurator methodsFor: 'geoSpec' stamp: 'hk 8/19/2014 17:47'!
simpleTerminalAGVArea
	^ '      			
	displayIn: TerminalArea size: 1000@500    	"defines the initial zoom"
	"AGVs"
		(ellipse:  agv01 size: 20@20 at: 100@150 label: agv1 color: lightGreen)
		(ellipse:  agv02 size: 20@20 at: 140@150 label: agv2 color: lightGreen)
		(ellipse:  agv03 size: 20@20 at: 160@150 label: agv3 color: lightGreen)
		(ellipse:  agv04 size: 20@20 at: 180@150 label: agv4 color: lightGreen)
		(ellipse:  agv05 size: 20@20 at: 200@150 label: agv5 color: lightGreen)
		(ellipse:  agv06 size: 20@20 at: 250@150 label: agv6 color: lightGreen)
		(ellipse:  agv07 size: 20@20 at: 280@150 label: agv7 color: lightGreen)
		(ellipse:  agv08 size: 20@20 at: 300@150 label: agv8 color: lightGreen)
		(ellipse:  agv09 size: 20@20 at: 320@150 label: agv9 color: lightGreen)
	"Lagerblock-Spuren"
		(rectangle:  laneLB1S1 size: 60@20 at: 360@60  bearing: 90   label: spur1 color: lightBlue)
		(rectangle:  laneLB1S2 size: 60@20 at: 380@60  bearing: 90   label: spur2 color: lightBlue)
		(rectangle:  laneLB1S3 size: 60@20 at: 400@60  bearing: 90   label: spur3 color: lightBlue)
		(rectangle:  laneLB1S4 size: 60@20 at: 420@60  bearing: 90   label: spur4 color: lightBlue)
		(rectangle:  laneLB2S1 size: 60@20 at: 760@60  bearing: 90   label: spur1 color: lightBlue)
		(rectangle:  laneLB2S2 size: 60@20 at: 780@60  bearing: 90   label: spur2 color: lightBlue)
		(rectangle:  laneLB2S3 size: 60@20 at: 800@60  bearing: 90   label: spur3 color: lightBlue)
		(rectangle:  laneLB2S4 size: 60@20 at: 820@60  bearing: 90   label: spur4 color: lightBlue)
	"CB-Spuren"	
		(rectangle:  laneCB1S1 size: 60@20 at: 360@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB1S2 size: 60@20 at: 360@260  label: spur2 color: lightGreen)
		(rectangle:  laneCB2S1 size: 60@20 at: 460@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB2S2 size: 60@20 at: 460@260  label: spur2 color: lightGreen)
		(rectangle:  laneCB3S1 size: 60@20 at: 760@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB3S2 size: 60@20 at: 760@260  label: spur2 color: lightGreen)
		(rectangle:  laneCB4S1 size: 60@20 at: 860@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB4S2 size: 60@20 at: 860@260  label: spur2 color: lightGreen)
	"Boundary"	
		(polygon: boundary [ 0@0  30@0  60@0  90@0  120@0  150@0  180@0  210@0  240@0  270@0  300@0 330@0  360@0  390@0  420@0  450@0  480@0  510@0  540@0  570@0  600@0  630@0  660@0  690@0  720@0  750@0  780@0  810@0  840@0  870@0  900@0  930@0  960@0  990@0 990@30  990@60  990@90  990@120  990@150  990@180  990@210  990@240  990@270   990@300  960@300  930@300  900@300  870@300  840@300  810@300  780@300  750@300  720@300  690@300  660@300  630@300  600@300  570@300  540@300  510@300  480@300  450@300  420@300  390@300  360@300  330@300  300@300  270@300  240@300  210@300  180@300  150@300  120@300  90@300  60@300  30@300  0@300 0@270   0@240   0@210   0@180   0@150   0@120   0@90   0@60   0@30   ] )
'! !

!AgvConfigurator methodsFor: 'accessing' stamp: 'hk 8/22/2014 18:54'!
visualisation
	^ visualisation! !

Object subclass: #AgvDispatcher
	instanceVariableNames: 'freeAgvs occAgvs freeTasks simulator running'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV-SIM'!

!AgvDispatcher commentStamp: '<historical>' prior: 0!
An AgvDispatcher dispatches transport tasks to agvs.
It allows to:  
- issue a new transport task
- start or stop the simulation
the AGV component ist (as a class) globally known.
the location of the agvs will be used by sophisticated optimizers.

Instance Variables
	freeAgvs:		<Dict of: Agv-Name and location>		free
	occAgvs:		<Dict of: Agv-Name and location>		occupied
	freeTasks:		<Dict of: Task>
	simulator:		<Block>
	running: 		<Boolean>				tested by the simulator block
							

Todo: =========================================
- Tests !

!AgvDispatcher methodsFor: 'accessing' stamp: 'hk 8/22/2014 18:02'!
freeAgvs
	^ (AGV allAGVs select: [ :v | v at: 2 ]) collect: [ :v | v at: 1 ].
 ! !

!AgvDispatcher methodsFor: 'interface-agv' stamp: 'hk 8/22/2014 18:00'!
getAgvs
	freeAgvs := self freeAgvs .
	occAgvs := self occAgvs 
	! !

!AgvDispatcher methodsFor: 'initialize-release' stamp: 'hk 7/4/2014 14:11'!
initialize 
	self stop.
	simulator := [ [true] whileTrue: [self step]] forkAt: Processor activePriority+1 .
	freeTasks := Dictionary new.
	occAgvs := Dictionary new.
	freeAgvs := Dictionary new.
	! !

!AgvDispatcher methodsFor: 'interface-agv' stamp: 'hk 8/22/2014 18:02'!
occAgvs
	^(AGV allAGVs select: [ :v | (v at: 2) not ]) collect: [ :v | v at: 1 ]
	! !

!AgvDispatcher methodsFor: 'initialize-release' stamp: 'hk 8/24/2014 15:44'!
release 
	AGV setArea: nil.
	simulator terminate. 
	simulator := nil.
! !

!AgvDispatcher methodsFor: 'simulation' stamp: 'hk 7/2/2014 20:36'!
start 
	running := true ! !

!AgvDispatcher methodsFor: 'simulation' stamp: 'hk 8/21/2014 13:51'!
step 
	running ifTrue: 	[AGV next].
	(Delay forMilliseconds: 400) wait ! !

!AgvDispatcher methodsFor: 'simulation' stamp: 'hk 7/2/2014 20:36'!
stop 
	running := false ! !

Object subclass: #AgvUI
	instanceVariableNames: 'configurator dispatcher chosenAgv chosenLane1 chosenLane2 lanes agvDropList freeAgvs window extent statusMorph'
	classVariableNames: 'AgvAnnouncer'
	poolDictionaries: ''
	category: 'AGV-SIM'!

!AgvUI commentStamp: '<historical>' prior: 0!
An AgvUI is the UI for the AGV simulation.

Instance Variables
	window:			<StandardWindow>	the GUI window
	extent:				<Point>				window extent
	statusMorph:		<TextMorph>			status line
	agvDropList:		<DropListMorph>		need this to be able to reset
	freeAgvs:			<Array of String>		for agv chooser
	chosenAgv:			<Integer>				for agv chooser
	lanes:				<Array of String>		for lane chooser
	chosenLane1:		<Integer>				for lane chooser (pick up)
	chosenLane2:		<Integer>				for lane chooser (destination))
	configurator:		<AgvConfigurator>		
	dispatcher:			<AgvDispatcher>		

Class variables
	AgvAnnouncer 		<Announcer>			for registered events!

!AgvUI class methodsFor: 'instance creation' stamp: 'hk 8/24/2014 15:29'!
announcer: anAnn 
	AgvAnnouncer := anAnn! !

!AgvUI class methodsFor: 'instance creation' stamp: 'hk 8/23/2014 14:33'!
on: anAgvConfigurator and: aAgvDispatcher
	^self new initializeWith: anAgvConfigurator with: aAgvDispatcher
! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/25/2014 11:47'!
agvToolbar

	   ^window newToolbar: 
		{window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv06' 'laneCB2S2' 'laneLB2S3')
			getEnabled: nil
			label: 'agv06:22-23'
			help: 'with agv06 from CB2S2 to LB2S3'. 
		window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv04' 'laneCB1S1' 'laneLB1S1')
			getEnabled: nil
			label: 'agv04:11-11'
			help: 'with agv04 from CB1S1 to LB1S1'. 
		window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv05' 'laneCB2S2' 'laneLB2S3')
			getEnabled: nil
			label: 'agv05:22-23'
			help: 'with agv05 from CB2S2 to LB2S3'. 
		window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv03' 'laneCB1S1' 'laneLB1S3')
			getEnabled: nil
			label: 'agv03:11-13'
			help: 'with agv03 from CB1S1 to LB1S3'. 
 }
! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenAgv
	^chosenAgv ifNil: [0]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenAgv: anIndex
	chosenAgv  := anIndex .
	self changed: #inputOK! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane1
	^chosenLane1 ifNil: [0]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane1: anIndex
	chosenLane1  := anIndex.
	self changed: #inputOK! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane2
	^chosenLane2 ifNil: [0]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane2: anIndex
	chosenLane2  := anIndex .
	self changed: #inputOK! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/24/2014 20:46'!
clearAgvDropList
	freeAgvs := nil.
	self chosenAgv: 0.	
	agvDropList updateListSelectionIndex.
	agvDropList updateList 
! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/24/2014 20:56'!
freeAgvs
	^ freeAgvs ifNil: [freeAgvs := dispatcher freeAgvs keys asOrderedCollection asSortedCollection]! !

!AgvUI methodsFor: 'initialize-release' stamp: 'hk 8/24/2014 15:46'!
initializeWindow
	| simTool agvTool agvTask y1 y2 |
	window := (StandardWindow labelled: 'AGV Simulation')
				model: configurator.
	y1 := (simTool := self simulationToolbar) minExtent y.
	y2 := y1 + (agvTool := self agvToolbar) minExtent y.
	agvTask := self taskEntry.
	window
		addMorph: simTool
		fullFrame: ((0 @ 0 corner: 1 @ 0) asLayoutFrame bottomOffset: y1).
	window addMorph: agvTool fullFrame: ((0 @ 0 corner: 1 @ 0) asLayoutFrame topOffset: y1;
			 bottomOffset: y2).
	window
		addMorph: agvTask
		fullFrame: ((0 @ 0 corner: 1 @ 0) asLayoutFrame topOffset: y2).
	window
		addMorph: (statusMorph := window newText: ' ')
		fullFrame: (LayoutFrame
				fractions: (0 @ 1 corner: 1 @ 1)
				offsets: (0 @ 50 negated corner: 0 @ 0)).
	window themeChanged.
	extent := window extent + (65 @ 90)! !

!AgvUI methodsFor: 'initialize-release' stamp: 'hk 8/24/2014 20:46'!
initializeWith: aConfigurator with: aDispatcher 
	configurator := aConfigurator .
	dispatcher := aDispatcher . 
	AgvAnnouncer on: TaskFinished do: [:ann | self clearAgvDropList; newStatus: ann asString ].
	AgvAnnouncer on: AgvOccupied do: [:ann | self newStatus: ann asString ].
	self initializeWindow.
	^self




! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
inputOK
	(self chosenAgv = 0) ifTrue: [ ^false ].
	(self chosenLane1 =  0) ifTrue: [ ^false ].
	(self chosenLane2 = 0) ifTrue: [ ^false ].
	(chosenLane1 = chosenLane2 ) ifTrue: [ ^false ].
	^true! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/24/2014 20:57'!
lanes
	^ lanes ifNil: [lanes := (configurator  visualisation allObjectNamesAndPositionsWith: 'lane*') keys asSortedCollection ]! !

!AgvUI methodsFor: 'status' stamp: 'hk 8/23/2014 15:53'!
newStatus: aStringOrText 
 	statusMorph contents: aStringOrText


! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:47'!
okButton
	^ window
		newButtonFor: self action: #submitTask getEnabled: #inputOK label: #Submit help: ''! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 8/23/2014 15:52'!
open 
	window openInWorld.
	window extent: extent


! !

!AgvUI methodsFor: 'initialize-release' stamp: 'hk 8/22/2014 20:16'!
release 
	dispatcher := nil.
	configurator release.
	configurator := nil ! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 19:56'!
simulationToolbar

	   ^window newToolbar: 
		{window
			newButtonFor: dispatcher
			getState: nil
			action: #start
			arguments: nil
			getEnabled: nil
			label: 'run'
			help: 'start simulation'. 
		window
			newButtonFor: dispatcher
			getState: nil
			action: #stop
			arguments: nil
			getEnabled: nil
			label: 'stop'
			help: 'stops simulation'.
		window
			newButtonFor: configurator
			getState: nil
			action: #inspect
			arguments: nil
			getEnabled: nil
			label: 'inspect'
			help: 'open inspector'.  }
! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 8/24/2014 20:18'!
submitTask
	AGV 	
		task: (freeAgvs at: chosenAgv )
		from:  (lanes  at: chosenLane1 )
		to: (lanes at: chosenLane2 ).
	self clearAgvDropList
! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 8/25/2014 11:50'!
task: agvNo from: lane1 to: lane2 
	AGV task: agvNo from: lane1  to: lane2.
	self clearAgvDropList ! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/23/2014 14:51'!
taskChoosers
	|   s d b |
	agvDropList :=  window
		newDropListFor: self
		list: #freeAgvs
		getSelected: #chosenAgv
		setSelected: #chosenAgv:
		help: 'choose on of the AGVs' .
	s :=  window
		newDropListFor: self
		list: #lanes
		getSelected: #chosenLane1
		setSelected: #chosenLane1:
		help: 'choose the pick up lane' .
	d :=  window
		newDropListFor: self
		list: #lanes
		getSelected: #chosenLane2
		setSelected: #chosenLane2:
		help: 'choose the delivery lane' .
	b := self okButton.
	^window newColumn: (Array with: agvDropList with: s with: d with: b)! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:51'!
taskEntry 
	^GroupboxMorph new
			addContentMorph: self taskEntryMorphs ;
			label: 'Select a new AGV Task:' ;
			vResizing: #spaceFill  ! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/23/2014 14:50'!
taskEntryMorphs
	^window newRow: (Array with: self taskLabels with: self taskChoosers )! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/23/2014 14:52'!
taskLabels
	|  a s d |
	
	a := window newLabel: 'AGV'.
	s := window newLabel: 'Pick Up'.
	d := window newLabel: 'destination'.
	a height: 26. s height: 26. d height: 26.
	^window newColumn: (Array with: a with: s with: d)! !

Object subclass: #Task
	instanceVariableNames: 'loadLane unloadLane stage id'
	classVariableNames: 'ID'
	poolDictionaries: ''
	category: 'AGV'!

!Task commentStamp: '<historical>' prior: 0!
A Task tells an AGV where to pick up and where to drop.
initialized with #empty

Instance Variables
	loadLane:		<Symbol>				pick up
	stage:			<#empty, #loaded, #finished>
	unloadLane:	<Symbol>				drop
	id: 				<String>				ref

Class Variables 	
	ID:				<Integer>				unique part of task id
					
Todo: =========================================
- Tests !

!Task class methodsFor: 'instance creation' stamp: 'hk 8/20/2014 18:09'!
id: partialID from: laneL to: laneU

	| t |
	t := self new.
	t id: partialID, '-', self nextID.	"make unique id"
	t loadLane: laneL.
	t unloadLane: laneU.
	^t
	! !

!Task class methodsFor: 'initialization' stamp: 'hk 8/20/2014 18:11'!
initialize
	ID := 42! !

!Task class methodsFor: 'private' stamp: 'hk 8/20/2014 18:05'!
nextID
	"must be unique"
	
	^(ID := ID + 1) asString! !

!Task methodsFor: 'testing' stamp: 'hk 7/2/2014 11:57'!
empty
	^stage = #empty! !

!Task methodsFor: 'testing' stamp: 'hk 7/2/2014 12:11'!
finished
	^ stage = #finished! !

!Task methodsFor: 'accessing' stamp: 'hk 7/2/2014 15:59'!
id 
	^id! !

!Task methodsFor: 'accessing' stamp: 'hk 8/20/2014 18:08'!
id: aString
	id := aString
	! !

!Task methodsFor: 'initialize-release' stamp: 'hk 7/2/2014 12:55'!
initialize
	stage := #empty! !

!Task methodsFor: 'accessing' stamp: 'hk 6/30/2014 11:20'!
loadLane 
	^loadLane! !

!Task methodsFor: 'accessing' stamp: 'hk 7/2/2014 12:55'!
loadLane: aString 
	loadLane := aString! !

!Task methodsFor: 'testing' stamp: 'hk 6/30/2014 11:23'!
loaded
	^ stage = #loaded! !

!Task methodsFor: 'simulation' stamp: 'hk 7/7/2014 14:48'!
next    
	self empty ifTrue: [^stage := #loaded ].
	self loaded ifTrue: [^stage := #finished]! !

!Task methodsFor: 'accessing' stamp: 'hk 6/30/2014 11:20'!
unloadLane 
	^unloadLane! !

!Task methodsFor: 'accessing' stamp: 'hk 7/2/2014 12:56'!
unloadLane: aString 
	unloadLane := aString! !
AGV initialize!
Task initialize!
